/* Copyright 2008 BORDEAUX I UNIVERSITY & INRIA 
**
** This file is part of the PaStiX parallel sparse matrix package.
**
** This software is governed by the CeCILL-C license under French law
** and abiding by the rules of distribution of free software. You can
** use, modify and/or redistribute the software under the terms of the
** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
** URL: "http://www.cecill.info".
** 
** As a counterpart to the access to the source code and rights to copy,
** modify and redistribute granted by the license, users are provided
** only with a limited warranty and the software's author, the holder of
** the economic rights, and the successive licensors have only limited
** liability.
** 
** In this respect, the user's attention is drawn to the risks associated
** with loading, using, modifying and/or developing or reproducing the
** software by the user in light of its specific status of free software,
** that may mean that it is complicated to manipulate, and that also
** therefore means that it is reserved for developers and experienced
** professionals having in-depth computer knowledge. Users are therefore
** encouraged to load and test the software's suitability as regards
** their requirements in conditions enabling the security of their
** systems and/or data to be ensured and, more generally, to use and
** operate it in the same conditions as regards security.
** 
** The fact that you are presently reading this means that you have had
** knowledge of the CeCILL-C license and that you accept its terms.
*/
#ifndef CSC_UTILS_H
#define CSC_UTILS_H

/*
  Function: csc_symgraph

  
  Modify the CSC to a symetric graph one.
  Don't use it on a lower symetric CSC 
  it would give you all the CSC upper + lower.
  
  External function

  Parameters: 
    n     - Number of columns/vertices
    ia	  - Starting index of each column in *ja* and *a*
    ja	  - Row index of each element
    a 	  - Value of each element,can be NULL    
    newn  - New number of column
    newia - Starting index of each column in *ja* and *a* 
    newja - Row index of each element
    newa  - Value of each element,can be NULL    

 */
int csc_symgraph(INT n, INT *ia, INT *ja, FLOAT *a, 
		 INT *newn, INT **newia, INT **newja, FLOAT **newa);



/*
  Function: csc_symgraph_int

  
  Modify the CSC to a symetric graph one.
  Don't use it on a lower symetric CSC 
  it would give you all the CSC upper + lower.
  
  Parameters: 
    n           - Number of columns/vertices
    ia	        - Starting index of each column in *ja* and *a*
    ja	        - Row index of each element
    a 	        - Value of each element,can be NULL    
    newn        - New number of column
    newia       - Starting index of each column in *ja* and *a* 
    newja       - Row index of each element
    newa        - Value of each element,can be NULL    
    malloc_flag - flag to indicate if function call is intern to pastix or extern.
 */
int csc_symgraph_int (INT n,     INT * ia,    INT * ja,    FLOAT * a, 
		      INT *newn, INT **newia, INT **newja, FLOAT **newa, 
		      int malloc_flag);

/*
  Function: csc_checksym

  Check if a CSC has a symetric graph.
  The CSC is supposed in fortran numerotation.

  Parameters:
    n  - Number or vertices/columns
    ia - Starting index of each column in *ja* and *a*
    ja - Row of each element
 */
int csc_checksym(INT n, INT *ia, INT *ja);


/** 
    Function: csc_noDiag
    
    Supress diagonal term.              
    After this call, *ja* can be reallocated to *ia[n] -1*.
    
    Parameters:
      n  - size of the matrix.
      ia - Index in *ja* and *a* of the first element of each column
      ja - row of each element
      a  - value of each element, can be set to NULL

    Returns:
      ia and ja tabulars modified.
*/
void csc_noDiag(INT baseval, INT n, INT *ia, INT *ja, FLOAT *a);

#endif /* CSC_UTILS_H */
