/* Copyright 2008 BORDEAUX I UNIVERSITY & INRIA 
**
** This file is part of the PaStiX parallel sparse matrix package.
**
** This software is governed by the CeCILL-C license under French law
** and abiding by the rules of distribution of free software. You can
** use, modify and/or redistribute the software under the terms of the
** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
** URL: "http://www.cecill.info".
** 
** As a counterpart to the access to the source code and rights to copy,
** modify and redistribute granted by the license, users are provided
** only with a limited warranty and the software's author, the holder of
** the economic rights, and the successive licensors have only limited
** liability.
** 
** In this respect, the user's attention is drawn to the risks associated
** with loading, using, modifying and/or developing or reproducing the
** software by the user in light of its specific status of free software,
** that may mean that it is complicated to manipulate, and that also
** therefore means that it is reserved for developers and experienced
** professionals having in-depth computer knowledge. Users are therefore
** encouraged to load and test the software's suitability as regards
** their requirements in conditions enabling the security of their
** systems and/or data to be ensured and, more generally, to use and
** operate it in the same conditions as regards security.
** 
** The fact that you are presently reading this means that you have had
** knowledge of the CeCILL-C license and that you accept its terms.
*/
/*
  File: errors.h

  Defines and macros used for error management.

  Authors:
    Mathieu Faverge - faverge@labri.fr
    Xavier   LACOSTE - lacoste@labri.fr
 */
#ifndef PASTIX_ERRORS_H
#define PASTIX_ERRORS_H
/*
  Group: Defines
*/
/*
  define: DUMP_FILENAME

  File where to dump parameters array if an error occur.
  
*/
#define DUMP_FILENAME "./dump.txt"

/* 
   Group: Macros
*/

/*
  Macro: ASSERT
  
  Prints an error and calls <EXIT> if a value is false.

  Parameters:
    expr   - Value to test.
    module - Module in which the test is performed.
*/
#define ASSERT(expr,module){\
    if((expr) == 0){\
      fprintf(stderr,"error in assert (line=%d,file=%s)\n",__LINE__,__FILE__);\
      EXIT((module),ASSERT_ERR);}}


/*
  Macro: ASSERTDBG
  
  If FLAG_ASSERT is defined, call <ASSERT>

  Parameters:
    expr   - Value to test.
    module - Module in which the test is performed.
*/
#ifdef FLAG_ASSERT
#define ASSERTDBG(expr,module) {ASSERT(expr,module)}
#else
#define ASSERTDBG(expr,module) {}
#endif

/*
  Macro: EXIT
  
  Set IPARM_ERROR_NUMBER  to module+error, dumps parameters and exit.

  Parameters: 
    module - Module where the error occurs.
    error  - Value to set IPARM_ERROR_NUMBER to.
*/
#define EXIT(module,error) {\
    set_iparm(reference_iparm, IPARM_ERROR_NUMBER, module+error);\
    api_dumparm(fopen(DUMP_FILENAME,"w+"),reference_iparm,reference_dparm);\
    exit(-1);}
/*
  Macro: RETURN
  
  If value is different from NO_ERR, sets IPARM_ERROR_NUMBER to module+error,
  dumps parameters and return value.

  Parameters: 
    value  - Value to test.
    module - Module where the error occurs.
    error  - Value to set <IPARM_ERROR_NUMBER> to.
*/
#define RETURN(value,module,error) {\
    if(error != NO_ERR){\
      set_iparm(reference_iparm, IPARM_ERROR_NUMBER, module+error);\
      api_dumparm(fopen(DUMP_FILENAME,"w+"),reference_iparm,reference_dparm);}\
    return(value);}
/*
  Macro: MALLOC_ERROR
  
  Prints an error message and call <EXIT> with MOD_UNKNOWN as module and
  ALLOC_ERR as error.
*/
#define MALLOC_ERROR(x) {\
    errorPrint("%s allocation (line=%d,file=%s)\n",(x),__LINE__,__FILE__);\
    EXIT(MOD_UNKNOWN,ALLOC_ERR);}

#endif /* PASTIX_ERRORS_H*/
