!  Copyright 2008 BORDEAUX I UNIVERSITY & INRIA ! **
! ** This file is part of the PaStiX parallel sparse matrix package.
! **
! ** This software is governed by the CeCILL-C license under French law
! ** and abiding by the rules of distribution of free software. You can
! ** use, modify and/or redistribute the software under the terms of the
! ** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
! ** URL: "http://www.cecill.info".
! ** 
! ** As a counterpart to the access to the source code and rights to copy,
! ** modify and redistribute granted by the license, users are provided
! ** only with a limited warranty and the software's author, the holder of
! ** the economic rights, and the successive licensors have only limited
! ** liability.
! ** 
! ** In this respect, the user's attention is drawn to the risks associated
! ** with loading, using, modifying and/or developing or reproducing the
! ** software by the user in light of its specific status of free software,
! ** that may mean that it is complicated to manipulate, and that also
! ** therefore means that it is reserved for developers and experienced
! ** professionals having in-depth computer knowledge. Users are therefore
! ** encouraged to load and test the software's suitability as regards
! ** their requirements in conditions enabling the security of their
! ** systems and/or data to be ensured and, more generally, to use and
! ** operate it in the same conditions as regards security.
! ** 
! ** The fact that you are presently reading this means that you have had
! ** knowledge of the CeCILL-C license and that you accept its terms.
! !   header: api.h
! 
!   Header file containing constants used in PaStiX and provided to users.
! 
!   Authors:
!     Mathieu Faverge - faverge@labri.fr
!     Xavier   Lacoste - lacoste@labri.fr
!     Pierre Ramet     - ramet@labri.fr
!  
!  !    enum: IPARM_ACCESS
!    
!    Integer parameters tabular accessors
! 
!    IPARM_MODIFY_PARAMETER      - Indicate if parameters have been set by user             Default: API_YES             IN
!    IPARM_START_TASK            - Indicate the first step to execute (see PaStiX steps)    Default: API_TASK_ORDERING   IN
!    IPARM_END_TASK              - Indicate the last step to execute (see PaStiX steps)     Default: API_TASK_CLEAN      IN
!    IPARM_VERBOSE               - Verbose mode (see Verbose modes)                         Default: API_VERBOSE_NO      IN
!    IPARM_DOF_NBR               - Degree of freedom per node                               Default: 1                   IN
!    IPARM_ITERMAX               - Maximum iteration number for refinement                  Default: 250                 IN
!    IPARM_MATRIX_VERIFICATION   - Check the input matrix                                   Default: API_NO              IN
!    IPARM_MC64                  - MC64 operation <pastix.h> IGNORE                         Default: 0                   IN
!    IPARM_ONLY_RAFF             - Refinement only                                          Default: API_NO              IN
!    IPARM_CSCD_CORRECT          - Indicate if the cscd has been redistributed after blend  Default: API_NO              IN
!    IPARM_NBITER                - Number of iterations performed in refinement   	  Default: -                   OUT
!    IPARM_TRACEFMT              - Trace format (see Trace modes)                           Default: API_TRACE_PICL      IN
!    IPARM_GRAPHDIST             - Specify if the given graph is distributed or not         Default: API_YES             IN
!    IPARM_AMALGAMATION_LEVEL    - Amalgamation level                                       Default: 5                   IN
!    IPARM_ORDERING              - Choose ordering                                          Default: API_ORDER_SCOTCH    IN
!    IPARM_DEFAULT_ORDERING      - Use default ordering parameters with scotch or metis     Default: API_YES             IN
!    IPARM_ORDERING_SWITCH_LEVEL - Ordering switch level    (see Scotch User's Guide)       Default: 120                 IN
!    IPARM_ORDERING_CMIN         - Ordering cmin parameter  (see Scotch User's Guide)       Default: 0                   IN
!    IPARM_ORDERING_CMAX         - Ordering cmax parameter  (see Scotch User's Guide)       Default: 100000              IN
!    IPARM_ORDERING_FRAT         - Ordering frat parameter  (see Scotch User's Guide)       Default: 8                   IN
!    IPARM_STATIC_PIVOTING       - Static pivoting                                          Default: -                   OUT
!    IPARM_METIS_PFACTOR         - Metis pfactor                                            Default: 0                   IN
!    IPARM_NNZEROS               - Number of Non Zeros in initial matrix                    Default: -                   OUT
!    IPARM_ALLOCATED_TERMS       - Number of Non Zeros in the factorized matrix             Default: -                   OUT
!    IPARM_BASEVAL               - Baseval used for the matrix                              Default: 0                   IN
!    IPARM_MIN_BLOCKSIZE         - Minimum blocksize                                        Default: 60                  OUT
!    IPARM_MAX_BLOCKSIZE         - Maximum blocksize                                        Default: 120                 OUT
!    IPARM_FACTORIZATION         - Factorization mode (see Factorization modes)             Default: API_FACT_LDLT       IN  
!    IPARM_CPU_BY_NODE           - Number of CPU per SMP node                               Default: 0                   IN
!    IPARM_BINDTHRD              - Thread binding mode (see Thread binding modes)           Default: API_BIND_AUTO       IN
!    IPARM_THREAD_NBR            - Number of thread per MPI process                         Default: 1                   IN
!    IPARM_DISTRIBUTION_LEVEL    - Distribution level IGNORE                                Default:                     IN
!    IPARM_LEVEL_OF_FILL         - Level of fill for incomplete factorization               Default: 1                   IN
!    IPARM_IO_STRATEGY           - IO strategy (see Check-points modes)                     Default: API_IO_NO           IN
!    IPARM_RHS_MAKING            - Right-hand-side making (see Rhight-hand-side modes)      Default: API_RHS_1           IN 
!    IPARM_REFINEMENT            - Refinement type (see Refinement modes)                   Default: API_RAF_GMRES       IN
!    IPARM_SYM                   - Symmetric matrix mode (see Symmetric modes)              Default: API_SYM_YES         IN
!    IPARM_INCOMPLETE            - Incomplete factorization                                 Default: API_NO              IN
!    IPARM_ABS                   - ABS (Automatic Blocksize Splitting)                      Default: API_NO              IN
!    IPARM_ESP                   - ESP (Enhanced Sparse Parallelism)                        Default: API_NO              IN
!    IPARM_GMRES_IM              - GMRES restart parameter                                  Default: 25                  IN
!    IPARM_FREE_CSCUSER          - Free user CSC                                            Default: API_CSC_PRESERVE    IN
!    IPARM_FREE_CSCPASTIX        - Free internal CSC (Use only without call to Refin. step) Default: API_CSC_PRESERVE    IN
!    IPARM_OOC_LIMIT             - Out of core memory limit (Mo)                            Default: 2000                IN  
!    IPARM_OOC_THREAD            - Out of core thread number IGNORE                         Default: 1                   IN
!    IPARM_OOC_ID                - Out of core run ID        IGNORE                         Default: -                   OUT
!    IPARM_NB_SMP_NODE_USED      - Number of SMP node used   IGNORE                         Default:                     IN
!    IPARM_THREAD_COMM_MODE      - Threaded communication mode (see Communication modes)    Default: API_THCOMM_DISABLED IN
!    IPARM_NB_THREAD_COMM        - Number of thread(s) for communication                    Default: 1                   IN
!    IPARM_FILL_MATRIX           - Initialize matrix coefficients (for test only)  IGNORE   Default:                     IN 
!    IPARM_INERTIA               - Return the inertia (symmetric matrix without pivoting)   Default: -                   OUT 
!    IPARM_ESP_NBTASKS           - Return the NUMBER OF TASKS GENERATED BY ESP              Default: -                   OUT 
!    IPARM_ERROR_NUMBER          - Return value                                             Default: -                   OUT
!    IPARM_SIZE                  - Iparm Size                IGNORE                         Default:                     IN 
#define IPARM_MODIFY_PARAMETER         1
#define IPARM_START_TASK               2
#define IPARM_END_TASK                 3
#define IPARM_VERBOSE                  4
#define IPARM_DOF_NBR                  5
#define IPARM_ITERMAX                  6
#define IPARM_MATRIX_VERIFICATION      7
#define IPARM_MC64                     8
#define IPARM_ONLY_RAFF                9
#define IPARM_CSCD_CORRECT             10
#define IPARM_NBITER                   11
#define IPARM_TRACEFMT                 12
#define IPARM_GRAPHDIST                13
#define IPARM_AMALGAMATION_LEVEL       14
#define IPARM_ORDERING                 15
#define IPARM_DEFAULT_ORDERING         16
#define IPARM_ORDERING_SWITCH_LEVEL    17
#define IPARM_ORDERING_CMIN            18
#define IPARM_ORDERING_CMAX            19
#define IPARM_ORDERING_FRAT            20
#define IPARM_STATIC_PIVOTING          21
#define IPARM_METIS_PFACTOR            22
#define IPARM_NNZEROS                  23
#define IPARM_ALLOCATED_TERMS          24
#define IPARM_BASEVAL                  25
#define IPARM_MIN_BLOCKSIZE            26
#define IPARM_MAX_BLOCKSIZE            27
#define IPARM_FACTORIZATION            31
#define IPARM_CPU_BY_NODE              33
#define IPARM_BINDTHRD                 34
#define IPARM_THREAD_NBR               35
#define IPARM_DISTRIBUTION_LEVEL       36
#define IPARM_LEVEL_OF_FILL            37
#define IPARM_IO_STRATEGY              38
#define IPARM_RHS_MAKING               39
#define IPARM_REFINEMENT               40
#define IPARM_SYM                      41
#define IPARM_INCOMPLETE               42
#define IPARM_ABS                      43
#define IPARM_ESP                      44
#define IPARM_GMRES_IM                 45
#define IPARM_FREE_CSCUSER             46
#define IPARM_FREE_CSCPASTIX           47
#define IPARM_OOC_LIMIT                48
#define IPARM_OOC_THREAD               49
#define IPARM_OOC_ID                   50
#define IPARM_NB_SMP_NODE_USED         51
#define IPARM_THREAD_COMM_MODE         52
#define IPARM_NB_THREAD_COMM           53
#define IPARM_FILL_MATRIX              54
#define IPARM_INERTIA                  55
#define IPARM_ESP_NBTASKS              56
#define IPARM_ERROR_NUMBER             64
#define IPARM_SIZE                     64
!  !    Enum: DPARM_ACCESS
!    
!    Floating point parameters tabular accossors
! 
!    DPARM_EPSILON_REFINEMENT - Epsilon for refinement          Default: 1e^{-12}         IN
!    DPARM_RELATIVE_ERROR     - Relative backward error         Default: -                OUT
!    DPARM_EPSILON_MAGN_CTRL  - Epsilon for magnitude control   Default: 1e^{-31}         IN
!    DPARM_ANALYZE_TIME       - Analize time                    Default: -                OUT
!    DPARM_PRED_FACT_TIME     - Predicted factorization time    Default: -                OUT
!    DPARM_FACT_TIME          - Factorization time              Default: -                OUT
!    DPARM_SOLV_TIME          - Solving time                    Default: -                OUT
!    DPARM_FACT_FLOPS         - Factorization flops             Default: -                OUT
!    DPARM_SOLV_FLOPS         - Solving flops                   Default: -                OUT
!    DPARM_RAFF_TIME          - Refinement time                 Default: -                OUT
!    DPARM_SIZE               - Dparm Size         IGNORE       Default: -                IN 
#define DPARM_EPSILON_REFINEMENT       6
#define DPARM_RELATIVE_ERROR           7
#define DPARM_EPSILON_MAGN_CTRL        11
#define DPARM_ANALYZE_TIME             19
#define DPARM_PRED_FACT_TIME           20
#define DPARM_FACT_TIME                21
#define DPARM_SOLV_TIME                22
#define DPARM_FACT_FLOPS               23
#define DPARM_SOLV_FLOPS               24
#define DPARM_RAFF_TIME                25
#define DPARM_SIZE                     64
! !   Enum: API_TASK
! 
!   PaStiX steps modes (index IPARM_START_TASK and IPARM_END_TASK)
! 
!   API_TASK_INIT       - Set default parameters
!   API_TASK_ORDERING   -	Ordering         
!   API_TASK_SYMBFACT   -	Symbolic factorization
!   API_TASK_ANALYSE    -	Tasks mapping and scheduling
!   API_TASK_NUMFACT    -	Numerical factorization   
!   API_TASK_SOLVE      -	Numerical solve        
!   API_TASK_REFINE     -	Numerical refinement
!   API_TASK_CLEAN      -	Clean           

#define API_TASK_INIT                  0
#define API_TASK_ORDERING              1
#define API_TASK_SYMBFACT              2
#define API_TASK_ANALYSE               3
#define API_TASK_NUMFACT               4
#define API_TASK_SOLVE                 5
#define API_TASK_REFINE                6
#define API_TASK_CLEAN                 7
! !   Enum: API_TASK_OLD
! 
!   API_TASK_SCOTCH     -	Ordering         
!   API_TASK_FAX        -	Symbolic factorization
!   API_TASK_BLEND      -	Tasks mapping and scheduling
!   API_TASK_SOPALIN    -	Numerical factorization   
!   API_TASK_UPDOWN     -	Numerical solve        
!   API_TASK_REFINEMENT -	Numerical refinement

#define API_TASK_SCOTCH                1
#define API_TASK_FAX                   2
#define API_TASK_BLEND                 3
#define API_TASK_SOPALIN               4
#define API_TASK_UPDOWN                5
#define API_TASK_REFINEMENT            6
!  !    Enum: API_VERBOSE
! 
!    Verbose modes (index IPARM_VERBOSE)
! 
!    API_VERBOSE_NOT - No display   
!    API_VERBOSE_NO  - Some displays
!    API_VERBOSE_YES - Many displays

#define API_VERBOSE_NOT                0
#define API_VERBOSE_NO                 1
#define API_VERBOSE_YES                2
! !   Enum: API_IO
! 
!   Check-points modes (index IPARM_IO)
!   
!   API_IO_NO   - No output/input   
!   API_IO_LOAD -	Load data during preprocessing steps
!   API_IO_SAVE -	Save data during preprocessing steps

#define API_IO_NO                      0
#define API_IO_LOAD                    1
#define API_IO_SAVE                    2
! !   Enum: API_RHS
! 
!   Right-hand-side modes (index IPARM_RHS)
! 
!   API_RHS_B - User's right hand side
!   API_RHS_1 - $ \forall i, X_i = 1 $
!   API_RHS_I - $ \forall i, X_i = i $
! 

#define API_RHS_B                      0
#define API_RHS_1                      1
#define API_RHS_I                      2
! !   Enum: API_RAF
!   
!   Refinement modes (index IPARM_REFINEMENT)
! 
!   API_RAF_GMRES - GMRES        
!   API_RAF_GRAD  - Conjugate Gradient (only for LU factorization)
!   API_RAF_PIVOT - Simple iterative algorithm ($LL^t$ or $LDL^t$ factorization)
!   

#define API_RAF_GMRES                  0
#define API_RAF_GRAD                   1
#define API_RAF_PIVOT                  1
! !   Enum: API_FACT
! 
!   Factorization modes (index IPARM_FACTORISATION_TYPE)
! 
!   API_FACT_LLT  - $LL^t$ Factorization 
!   API_FACT_LDLT - $LDL^t$ Factorization
!   API_FACT_LU   - $LU$ Factorization
! 

#define API_FACT_LLT                   0
#define API_FACT_LDLT                  1
#define API_FACT_LU                    2
! !   Enum: API_SYM
! 
!   Symetric modes (index IPARM_SYM)
! 
!   API_SYM_YES - Symmetric matrix
!   API_SYM_NO  - Non Symmetric matrix
! 

#define API_SYM_YES                    0
#define API_SYM_NO                     1
! !   Enum: API_ERASE_CSC
! 
!   CSC Management modes (index IPARM_FREE_CSCUSER and IPARM_FREE_CSCPASTIX)
! 
!   API_CSC_PRESERVE - Do not free the CSC
!   API_CSC_FREE     - Free the CSC when not needed anymore

#define API_CSC_PRESERVE               0
#define API_CSC_FREE                   1
! !   Enum: API_COMM_THREAD_MODE
! 
!   Comunication modes (index IPARM_NB_THREAD_COMM)
! 
!   API_THCOMM_DISABLED - No thread dedicated to communications        
!   API_THCOMM_ONE      -	One thread dedicated to communications
!   API_THCOMM_DEFINED  -	Given by IPARM_NB_THREAD_COMM
!   API_THCOMM_NBPROC   -	One communication thread per computation thread

#define API_THCOMM_DISABLED            0
#define API_THCOMM_ONE                 1
#define API_THCOMM_DEFINED             2
#define API_THCOMM_NBPROC              3
! !   Enum: API_BIND_MODE
! 
!   Thread-binding modes (index IPARM_BINTHRD)
! 
!   API_BIND_NO   - Do not bind thread
!   API_BIND_AUTO	- Default binding                            
!   API_BIND_TAB  - Use vector given by pastix_setBind

#define API_BIND_NO                    0
#define API_BIND_AUTO                  1
#define API_BIND_TAB                   2
! !   Enum: API_BOOLEAN
! 
!   Boolean modes (All boolean except IPARM_SYM)
! 
!   API_NO  - No
!   API_YES - Yes

#define API_NO                         0
#define API_YES                        1
! !   Enum: API_TRACEFMT
! 
!   Trace modes (index IPARM_TRACEFMT)
! 
!   API_TRACE_PICL       - Use PICL trace format
!   API_TRACE_PAJE       - Use Paje trace format
!   API_TRACE_HUMREAD    - Use Text trace format
!   API_TRACE_UNFORMATED - Use Unformated trace format

#define API_TRACE_PICL                 0
#define API_TRACE_PAJE                 1
#define API_TRACE_HUMREAD              2
#define API_TRACE_UNFORMATED           3
! !   Enum: API_ORDER
! 
!   Ordering modes (index IPARM_ORDERING)
! 
!   API_ORDER_SCOTCH   - Use Scotch ordering
!   API_ORDER_METIS    - Use Metis ordering
!   API_ORDER_PERSONAL - Use given permutation (resp. reverse permutation) array
!   API_ORDER_LOAD     - Load ordering from disk

#define API_ORDER_SCOTCH               0
#define API_ORDER_METIS                1
#define API_ORDER_PERSONAL             2
#define API_ORDER_LOAD                 3
! !   Enum: MODULES
! 
!   Module Identification number.
! 
!   If an error occurs, error value is set to 
!   MODULE + EER_NUMBER.
! 
!   User can catch error by computing iparm[IPARM_ERROR_NUMBER]%100.
! 
!   MODULE can be catch by computing iparm[IPARM_ERROR_NUMBER] - iparm[IPARM_ERROR_NUMBER]%100.
!   
!   MOD_UNKNOWN - Unknown module
!   MOD_SOPALIN - Numerical factorisation module
!   MOD_BLEND   - Analysing module
!   MOD_SCOTCH  - Scotch module 
!   MOD_FAX     - Symbolic factorisation module
!   MOD_ORDER   - Order module
!   MOD_COMMON  - Common module
!   MOD_SI      - 
!   MOD_GRAPH   - Graph module
!   MOD_SYMBOL  - Symbol structure module
!   MOD_KASS    - Kass module
!   MOD_BUBBLE  - Bubble
! 
#define MOD_UNKNOWN                    0
#define MOD_SOPALIN                    100
#define MOD_BLEND                      200
#define MOD_SCOTCH                     300
#define MOD_FAX                        400
#define MOD_ORDER                      500
#define MOD_COMMON                     600
#define MOD_SI                         700
#define MOD_GRAPH                      800
#define MOD_SYMBOL                     900
#define MOD_KASS                       1000
#define MOD_BUBBLE                     1100
!  Enum: ERR_NUMBERS! 
!    Error Numbers 
! 
!    NO_ERR             - No error
!    UNKNOWN_ERR        - Unknown error
!    ALLOC_ERR          - Allocation error
!    ASSERT_ERR         - Error in one assertion
!    NOTIMPLEMENTED_ERR - Not implemented feature
!    OUTOFMEMORY_ERR    - Not enough memory (OOC)
!    THREAD_ERR         - Error with threads
!    INTERNAL_ERR       - Internal error
!    BADPARAMETER_ERR   - Bad parameters given
!    FILE_ERR           - Error in In/Out operations
!    BAD_DEFINE_ERROR   - Error with defines during compilation
#define NO_ERR                         0
#define UNKNOWN_ERR                    1
#define ALLOC_ERR                      2
#define ASSERT_ERR                     3
#define NOTIMPLEMENTED_ERR             4
#define OUTOFMEMORY_ERR                5
#define THREAD_ERR                     6
#define INTERNAL_ERR                   7
#define BADPARAMETER_ERR               8
#define FILE_ERR                       9
#define BAD_DEFINE_ERROR               10
#define INTEGER_TYPE                   11

#define MTX_ISSYM                      0
#define MTX_ISCOM                      0
#define MTX_ISRHX                      0
#define MTX_ISRHS                      0
