/* Copyright 2008 BORDEAUX I UNIVERSITY & INRIA 
**
** This file is part of the PaStiX parallel sparse matrix package.
**
** This software is governed by the CeCILL-C license under French law
** and abiding by the rules of distribution of free software. You can
** use, modify and/or redistribute the software under the terms of the
** CeCILL-C license as circulated by CEA, CNRS and INRIA at the following
** URL: "http://www.cecill.info".
** 
** As a counterpart to the access to the source code and rights to copy,
** modify and redistribute granted by the license, users are provided
** only with a limited warranty and the software's author, the holder of
** the economic rights, and the successive licensors have only limited
** liability.
** 
** In this respect, the user's attention is drawn to the risks associated
** with loading, using, modifying and/or developing or reproducing the
** software by the user in light of its specific status of free software,
** that may mean that it is complicated to manipulate, and that also
** therefore means that it is reserved for developers and experienced
** professionals having in-depth computer knowledge. Users are therefore
** encouraged to load and test the software's suitability as regards
** their requirements in conditions enabling the security of their
** systems and/or data to be ensured and, more generally, to use and
** operate it in the same conditions as regards security.
** 
** The fact that you are presently reading this means that you have had
** knowledge of the CeCILL-C license and that you accept its terms.
*/
/*
  File: trace.h                                 

  trace instruction for different format

  Authors: 
    Mathieu Faverge - faverge@labri.fr

  Dates: 
    Varsion 0.0 - from 12 apr 2008
                  to   12 apr 2008
*/
#ifndef TRACE_H
#define TRACE_H

#ifdef TRACE_SOPALIN

typedef enum API_TRACEFMT TraceFmt_t;

typedef enum Trace_State {
  STATE_IDLE         =  0,
  STATE_WAITLOC      =  1,
  STATE_WAITREM      =  2,
  STATE_WAITTSK	     =  3,
  STATE_FACTOINIT    =  4,
  STATE_FACTOCLEAN   =  5,
  STATE_FACTO	     =  6,
  STATE_UPDOINIT     =  7,
  STATE_UPDOCLEAN    =  8,
  STATE_DOWN	     =  9,
  STATE_UP	     = 10,
  STATE_RAFF	     = 11,
  STATE_COMPUTE	     = 12,
  STATE_BLOCK	     = 13,
  STATE_DIAG	     = 14,
  STATE_COMP1D	     = 15,
  STATE_E1	     = 16,
  STATE_E2	     = 17,
  STATE_ADD	     = 18,
  STATE_SENDF	     = 19,
  STATE_SENDB	     = 20,
  STATE_RECVF	     = 21,
  STATE_RECVB	     = 22,
  STATE_RECVDOWN     = 23,
  STATE_RECVUP	     = 24,
  STATE_COMMG	     = 25,
  STATE_NBSTATES     = 26,
  STATE_ALLOC	     = 27,
  STATE_FREE         = 28
} Trace_State_t;       

typedef enum Trace_Comm {
  COMM_FANIN = 0,
  COMM_BLOCK,
  COMM_DOWN,
  COMM_UP,
  COMM_NBTYPECOMM
} Trace_Comm_t;

/*
 * Common part :
 *    - TraceFmt_t  fmt     : Format de trace
 *    - FILE       *file    : fichier de trace
 *    - double      time    : timestamp
 *    - INT         procnum
 *    - INT         thrdnum
 * State :
 *    - INT           level : niveau de trace (+il est grand plus c'est dtaill)
 *    - Trace_State_T state : Etat dans lequel se retrouve le proc
 *    - INT           id    : identifiant de la tche
 * Comm :
 *    - INT          dest/src : destinataire ou source du message
 *    - Trace_Comm_t Type     : type de communication
 *    - INT          id       : identifiant du message
 *    - INT          size     : taille du message
 */   

extern volatile INT idg;

#define trace_start(file, time, procnum, thrdnum) \
  fprintf(file, "%9.9lf %ld %ld 0 -1 0\n", (double)time, (long)procnum, (long)thrdnum);
#define trace_finish(file, time, procnum, thrdnum) \
  fprintf(file, "%9.9lf %ld %ld 0 -1 1\n", (double)time, (long)procnum, (long)thrdnum);
#define trace_send(file, time, procnum, thrdnum, dest, type, id, size, idreq) \
  {									\
  INT idl = idg++;							\
  fprintf(file, "%9.9lf %ld %ld %ld 2 %ld %ld %ld\n",			\
	  (double)time, (long)procnum, (long)thrdnum, (long)dest, (long)type, (long)id, (long)idl);	\
  *idreq = idl;								\
  }
#define trace_recv(file, time, procnum, thrdnum, src, type, id, size, idreq) \
  fprintf(file, "%9.9lf %ld %ld %ld 3 %ld %ld %ld\n",			\
	  (double)time, (long)procnum, (long)thrdnum, (long)src, (long)type, (long)id, (long)idreq);

#define trace_begin_task(file, time, procnum, thrdnum, level, state, id) \
  {									\
  if (level != 2)							\
    fprintf(file, "%9.9lf %ld %ld %ld 0 %ld %ld\n",				\
	    (double)time, (long)procnum, (long)thrdnum, (long)level, (long)state, (long)id);\
  }

#define trace_end_task(file, time, procnum, thrdnum, level, state, id)\
  {									\
    if (level != 2)							\
      fprintf(file, "%9.9lf %ld %ld %ld 1 %ld %ld\n",			\
	      (double)time, (long)procnum, (long)thrdnum, (long)level, (long)state, (long)id); \
  }

#define trace_begin_task1(file, time, procnum, thrdnum, state, id2, task) \
  {									\
    fprintf(file, "%9.9lf %ld %ld 1 0 %ld %ld %ld %ld %ld %ld\n",	\
	    (double)time, (long)procnum, (long)thrdnum, (long)state,	\
	    (long)task.id, (long)task.threadid, (long)task.fcandnum,	\
	    (long)task.lcandnum, (long)task.cand			\
	    );								\
  }

#define trace_begin_task2(file, time, procnum, thrdnum, state, id, fcand, lcand, cand) \
  {									\
    fprintf(file, "%9.9lf %ld %ld 1 0 %ld %ld %ld %ld %ld %ld\n",	\
	    (double)time, (long)procnum, (long)thrdnum, (long)state,	\
	    (long)id,      (long)0, (long)(fcand),			\
	    (long)(lcand), (long)(cand)					\
	    );								\
  }

#define trace_malloc(file, time, procnum, state, memory)	\
  fprintf(file, "%9.9lf %ld 0 0 4 %ld %f\n",				\
	  (double)time, (long)procnum, (long)state, (double)memory/(1<<20));

#else /* TRACE_SOPALIN */

#define trace_start(...)      {}
#define trace_finish(...)     {}
#define trace_begin_task(...) {}
#define trace_end_task(...)   {}
#define trace_begin_task1(...){}
#define trace_begin_task2(...){}
#define trace_send(...)       {}
#define trace_recv(...)       {}
#define trace_malloc(...)     {}

#endif /* TRACE_SOPALIN */
#endif /* TRACE_H */
